using System.Collections;

namespace InfiniTec.SystemUtilities
{
	/// <summary>
	/// This compares two ByteBags or one <see cref="ByteBag"/> and an <see cref="System.Int64"/> value.
	/// See the <see cref="ByteBagComparer.Compare"/> method for more information.
	/// </summary>
	internal class ByteBagComparer: IComparer
	{
		/// <summary>
		/// Initializes a new isntance of this class.
		/// </summary>
		public ByteBagComparer()
		{
		}

		/// <summary>
		/// Compares the size of a <see cref="ByteBag"/> with the given size.
		/// </summary>
		/// <param name="bag">A bag containing a byte buffer</param>
		/// <param name="size">Size</param>
		/// <returns>
		/// <p>0, if the the size of the buffer within the bag is equal to the given size</p>
		/// <p>1, if the size of the buffer exceeds the given size</p>
		/// <p>-1, if the size of the buffer is smaller than the given size</p>
		/// </returns>
		private int Compare(ByteBag bag, long size)
		{
			return bag.Size.CompareTo(size);
		}

		/// <summary>
		/// Compares the size of two Bytebags or one byte bag with a given size.
		/// </summary>
		/// <remarks>
		/// If both objects are <see cref="ByteBag"/> objects, the sizes of the contained
		/// buffers are compared. If one argument is an Int64 value, the size the buffer
		/// in the other argument is compared to the specified size.
		/// </remarks>
		/// <param name="x">Either a <see cref="ByteBag"/> object or an <see cref="System.Int64"/> value</param>
		/// <param name="y">Either a <see cref="ByteBag"/> object or an <see cref="System.Int64"/> value</param>
		/// <returns>
		/// <p>0, if the size of x and y are equal.</p>
		/// <p>1, if the size of x exceeds the size of y</p>
		/// <p>-1, if the size of x is smaller than the size of y</p>
		/// </returns>
		public int Compare(object x, object y)
		{
			if ((x == null) && (y == null)) return 0;
			if (x == null) return -1;
			if (y == null) return 1;

			if ((x is ByteBag) && (y is ByteBag)) return ((ByteBag) x).CompareTo((ByteBag) y);

			if ((x is ByteBag) && (y is long)) return Compare((ByteBag) x, (long) y);
			if ((x is long) && (y is ByteBag)) return -Compare((ByteBag) y, (long) x);

			return 0;
		}
	}
}
